import Foundation

// MARK: -
/** Maps client-side IDs to server-side IDs. */
protocol IDMap: AnyObject {

  /// Generates a log user ID which is used to track the the current
  /// session without exposing the underlying user ID.
  func logUserID() -> ID
  
  /// Generates a new session ID.
  func sessionID() -> ID

  /// Generates a new impression ID.
  func impressionID() -> ID

  /// Generates a new click ID.
  func actionID() -> ID
  
  /// Generates a new view ID.
  func viewID() -> ID
}

protocol IDMapSource {
  var idMap: IDMap { get }
}

// MARK: -
final class DefaultIDMap: IDMap {

  func logUserID() -> ID { .autogenerated(value: UUID().uuidString) }
  
  func sessionID() -> ID { .autogenerated(value: UUID().uuidString) }

  func impressionID() -> ID { .autogenerated(value: UUID().uuidString) }
  
  func actionID() -> ID { .autogenerated(value: UUID().uuidString) }
  
  func viewID() -> ID { .autogenerated(value: UUID().uuidString) }
}
